﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Text.RegularExpressions;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 키워드 리스트 딕셔너리
        /// </summary>
        KeywordListDictionary keywordListDictionary = null;

        /// <summary>
        /// 키워드 리스트
        /// </summary>
        /// <remarks>키워드 리스트 박스 데이터를 표시합니다.</remarks>
        KeywordList keywordList = null;

        /// <summary>
        /// 이전 키워드
        /// </summary>
        private Keyword currentKeyword = null;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.textRichTextBox.Text = @"
using System.Collections.Generic;
 
/// <summary>
/// 문자열 길이 비교자
/// </summary>
public class StringLengthComparer : IComparer<string>
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Private
 
    #region 비교하기 - Compare(source1, source2)
 
    /// <summary>
    /// 비교하기
    /// </summary>
    /// <param name=""source1"">소스 문자열 1</param>
    /// <param name=""source2"">소스 문자열 2</param>
    /// <returns>비교 결과</returns>
    public int Compare(string source1, string source2)
    {
        if(source1.Length == source2.Length)
        {
            return 0;
        }
        else if(source1.Length > source2.Length)
        {
            return 1;
        }
        else
        {
            return -1;
        }
    }
 
    #endregion
}
";

            #region 이벤트를 설정한다.

            this.searchButton.Click                  += searchButton_Click;
            this.keywordListBox.SelectedIndexChanged += keywordListBox_SelectedIndexChanged;
            this.deleteItemButton.Click              += deleteItemButton_Click;

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 조회 버튼 클릭시 처리하기 - searchButton_Click(sender, e)

        /// <summary>
        /// 조회 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void searchButton_Click(object sender, EventArgs e)
        {
            #region 정규식을 구한다.

            string regularExpression = this.regularExpressionTextBox.Text.Trim();

            if(string.IsNullOrEmpty(regularExpression))
            {
                MessageBox.Show(this, "정규식을 입력해 주시기 바랍니다.", "INFORMATION", MessageBoxButtons.OK, MessageBoxIcon.Information);

                this.regularExpressionTextBox.Focus();

                return;
            }

            #endregion
            #region 텍스트를 구한다.

            string text = this.textRichTextBox.Text.Trim();

            if(string.IsNullOrEmpty(text))
            {
                MessageBox.Show(this, "텍스트를 입력해 주시기 바랍니다.", "INFORMATION", MessageBoxButtons.OK, MessageBoxIcon.Information);

                this.textRichTextBox.Focus();

                return;
            }

            #endregion
            #region 텍스트 리치 텍스트 박스 데이터 스타일을 지운다.

            this.textRichTextBox.SelectAll();

            this.textRichTextBox.SelectionFont      = this.textRichTextBox.Font;
            this.textRichTextBox.SelectionColor     = this.textRichTextBox.ForeColor;
            this.textRichTextBox.SelectionBackColor = this.textRichTextBox.BackColor;

            this.textRichTextBox.Select(0, 0);

            #endregion
            #region 키워드 리스트 딕셔너리를 설정한다.

            if(this.keywordListDictionary != null)
            {
                this.keywordListDictionary.ClearData();
            }

            this.keywordListDictionary = GetKeywordListDictionary(regularExpression, text);

            #endregion
            #region 키워드 리스트 박스 데이터를 지운다.

            this.keywordListBox.Items.Clear();

            #endregion
            #region 키워드 리스트를 설정한다.

            if(this.keywordList != null)
            {
                this.keywordList.Clear();
            }

            this.keywordList = GetKeywordList(this.keywordListDictionary);

            #endregion
            #region 키워드 리스트 박스 데이터를 추가한다.

            foreach(Keyword keyword in this.keywordList)
            {
                this.keywordListBox.Items.Add(keyword);
            }

            #endregion
            #region 텍스트 리치 텍스트 박스에서 해당 단어를 표시한다.

            foreach(KeyValuePair<string, KeywordList> keyValuePair in this.keywordListDictionary)
            {
                KeywordList keywordList = keyValuePair.Value;

                foreach(Keyword keyword in keywordList)
                {
                    this.textRichTextBox.Select(keyword.Index, keyword.Length);
 
                    this.textRichTextBox.SelectionColor     = Color.Red;
                    this.textRichTextBox.SelectionBackColor = Color.Yellow;
                }
            }

            #endregion
        }

        #endregion
        #region 키워드 리스트 박스 선택 인덱스 변경시 처리하기 - keywordListBox_SelectedIndexChanged(sender, e)

        /// <summary>
        /// 키워드 리스트 박스 선택 인덱스 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void keywordListBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if(this.currentKeyword != null)
            {
                this.textRichTextBox.Select(this.currentKeyword.Index, this.currentKeyword.Length);

                this.textRichTextBox.SelectionFont = this.textRichTextBox.Font;
            }

            if(this.keywordListBox.SelectedIndex == -1)
            {
                this.currentKeyword = null;
            }
            else
            {
                Keyword keyword = this.keywordListBox.SelectedItem as Keyword;

                this.textRichTextBox.Select(keyword.Index, keyword.Length);

                this.textRichTextBox.SelectionFont = new Font(this.textRichTextBox.Font, FontStyle.Bold);

                this.currentKeyword = keyword;
            }
        }

        #endregion
        #region 항목 삭제 버튼 클릭시 처리하기 - deleteItemButton_Click(sender, e)

        /// <summary>
        /// 항목 삭제 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void deleteItemButton_Click(object sender, EventArgs e)
        {
            if(this.keywordListBox.SelectedIndex == -1)
            {
                return;
            }

            Keyword keyword = this.keywordListBox.Items[this.keywordListBox.SelectedIndex] as Keyword;

            this.textRichTextBox.Select(keyword.Index, keyword.Length);

            this.textRichTextBox.SelectionColor     = this.textRichTextBox.ForeColor;
            this.textRichTextBox.SelectionBackColor = this.textRichTextBox.BackColor;

            this.keywordListBox.Items.RemoveAt(this.keywordListBox.SelectedIndex);


            this.keywordList.Remove(keyword);

            this.keywordListDictionary.RemoveItem(keyword);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 키워드 리스트 딕셔너리 구하기 - GetKeywordListDictionary(regularExpression, text)

        /// <summary>
        /// 키워드 리스트 딕셔너리 구하기
        /// </summary>
        /// <param name="regularExpression">정규식</param>
        /// <param name="text">텍스트</param>
        private KeywordListDictionary GetKeywordListDictionary(string regularExpression, string text)
        {
            KeywordListDictionary dictionary = new KeywordListDictionary();

            Regex regex = new Regex(regularExpression);

            MatchCollection matchCollection = regex.Matches(text);

            foreach(Match match in matchCollection)
            {
                Keyword keyword = new Keyword(match);

                dictionary.AddItem(keyword);
            }

            return dictionary;
        }

        #endregion
        #region 키워드 리스트 구하기 - GetKeywordList(dictionary)

        /// <summary>
        /// 키워드 리스트 구하기
        /// </summary>
        /// <param name="dictionary">딕셔너리</param>
        /// <returns>키워드 리스트</returns>
        private KeywordList GetKeywordList(KeywordListDictionary dictionary)
        {
            KeywordList targetList = new KeywordList();

            foreach(KeyValuePair<string, KeywordList> keyValuePair in dictionary)
            {
                KeywordList keywordList = keyValuePair.Value;

                foreach(Keyword keyword in keywordList)
                {
                    targetList.Add(keyword);
                }
            }

            return targetList;
        }

        #endregion
        #region 키워드 리스트 박스 데이터 지우기 - ClearKeywordListBoxData()

        /// <summary>
        /// 키워드 리스트 박스 데이터 지우기
        /// </summary>
        private void ClearKeywordListBoxData()
        {
            this.keywordListBox.Items.Clear();
        }

        #endregion
    }
}